@@ -20,9 +20,12 @@ from neutron.callbacks import events
 from neutron.callbacks import registry
 from neutron.callbacks import resources
 from neutron.common import constants as const
+from neutron.common import exceptions as n_exc
 from neutron import context as n_context
+from neutron.db import external_net_db
 from neutron.db import l3_db
 from neutron.db import models_v2
+from neutron.debug import debug_agent
 from neutron.extensions import portbindings
 from neutron.i18n import _LE
 from neutron import manager
@@ -33,12 +36,22 @@ from oslo_log import log as logging
 from networking_bagpipe.agent.bgpvpn import rpc_client
 
 from networking_bgpvpn.neutron.db import bgpvpn_db
+from networking_bgpvpn.neutron.extensions import bgpvpn as bgpvpn_ext
+from networking_bgpvpn.neutron.services.common import constants
 from networking_bgpvpn.neutron.services.common import utils
 from networking_bgpvpn.neutron.services.service_drivers import driver_api
 
 LOG = logging.getLogger(__name__)
 
 
+BAGPIPE_DRIVER_NAME = "bagpipe"
+
+
+class BGPVPNExternalNetAssociation(n_exc.NeutronException):
+    message = _("driver does not support associating an external"
+                "network to a BGPVPN")
+
+
 def get_network_info_for_port(context, port_id):
     """Get MAC, IP and Gateway IP addresses informations for a specific port"""
 
@@ -128,6 +141,15 @@ def get_bgpvpns_of_router_assocs_by_network(context, net_id):
         return
 
 
+def network_is_external(context, net_id):
+    try:
+        context.session.query(external_net_db.ExternalNetwork).filter_by(
+            network_id=net_id).one()
+        return True
+    except exc.NoResultFound:
+        return False
+
+
 class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
     """BGPVPN Service Driver class for BaGPipe"""
@@ -276,6 +298,19 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
         return [self.bgpvpn_db._make_bgpvpn_dict(bgpvpn) for bgpvpn in
                 get_bgpvpns_of_router_assocs_by_network(context, network_id)]
 
+    def _common_precommit_checks(self, bgpvpn):
+        # No support yet for specifying route distinguishers
+        if bgpvpn.get('route_distinguishers', None):
+            raise bgpvpn_ext.BGPVPNRDNotSupported(driver=BAGPIPE_DRIVER_NAME)
+
+    def create_bgpvpn_precommit(self, context, bgpvpn):
+        # Only l3 type is supported
+        if bgpvpn['type'] != constants.BGPVPN_L3:
+            raise bgpvpn_ext.BGPVPNTypeNotSupported(driver=BAGPIPE_DRIVER_NAME,
+                                                    type=bgpvpn['type'])
+
+        self._common_precommit_checks(bgpvpn)
+
     def delete_bgpvpn_postcommit(self, context, bgpvpn):
         for net_id in bgpvpn['networks']:
             if get_network_ports(context, net_id):
@@ -284,6 +319,9 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                     context,
                     self._format_bgpvpn(bgpvpn, net_id))
 
+    def update_bgpvpn_precommit(self, context, old_bgpvpn, bgpvpn):
+        self._common_precommit_checks(bgpvpn)
+
     def update_bgpvpn_postcommit(self, context, old_bgpvpn, bgpvpn):
         (added_keys, removed_keys, changed_keys) = (
             utils.get_bgpvpn_differences(bgpvpn, old_bgpvpn))
@@ -303,6 +341,10 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                         self._format_bgpvpn(bgpvpn, net_id)
                     )
 
+    def create_net_assoc_precommit(self, context, net_assoc):
+        if network_is_external(context, net_assoc['network_id']):
+            raise BGPVPNExternalNetAssociation()
+
     def create_net_assoc_postcommit(self, context, net_assoc):
         if get_network_ports(context, net_assoc['network_id']):
             bgpvpn = self.get_bgpvpn(context, net_assoc['bgpvpn_id'])
@@ -336,7 +378,23 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
 
         return port.get(portbindings.HOST_ID)
 
-    def notify_port_updated(self, context, port_id):
+    def _ignore_port(self, context, port):
+        if (port['device_owner'].startswith(const.DEVICE_OWNER_NETWORK_PREFIX)
+                and not port['device_owner'] in
+                (debug_agent.DEVICE_OWNER_COMPUTE_PROBE,
+                 debug_agent.DEVICE_OWNER_NETWORK_PROBE)):
+            LOG.info("Port %s owner is network:*, we'll do nothing",
+                     port['id'])
+            return True
+
+        if network_is_external(context, port['network_id']):
+            LOG.info("Port %s is on an external network, we'll do nothing",
+                     port['id'])
+            return True
+
+        return False
+
+    def notify_port_updated(self, context, port_id, original_port):
         LOG.info("notify_port_updated on port %s", port_id)
 
         port = self._get_port(context, port_id)
@@ -344,14 +402,18 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
         port_bgpvpn_info = {'id': port['id'],
                             'network_id': port['network_id']}
 
-        if port['device_owner'] == const.DEVICE_OWNER_DHCP:
-            LOG.info("Port %s is DHCP, ignoring", port['id'])
+        if self._ignore_port(context, port):
             return
 
         agent_host = self._get_port_host(context, port)
 
-        if port['status'] == const.PORT_STATUS_ACTIVE:
-            LOG.info("notify_port_updated, active")
+        original_port_status = "not present in notification"
+        if original_port:
+            original_port_status = original_port['status']
+
+        if (port['status'] == const.PORT_STATUS_ACTIVE and
+                original_port_status != const.PORT_STATUS_ACTIVE):
+            LOG.info("notify_port_updated, port became ACTIVE")
             bgpvpn_network_info = (
                 self._retrieve_bgpvpn_network_info_for_port(context, port)
             )
@@ -369,13 +431,15 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
                 # of any BGPVPN port bound.
                 pass
 
-        elif port['status'] == const.PORT_STATUS_DOWN:
-            LOG.info("notify_port_updated, down")
+        elif (port['status'] == const.PORT_STATUS_DOWN and
+                original_port_status != const.PORT_STATUS_DOWN):
+            LOG.info("notify_port_updated, port became DOWN")
             self.agent_rpc.detach_port_from_bgpvpn(context,
                                                    port_bgpvpn_info,
                                                    agent_host)
         else:
-            LOG.info("new port status is %s, no action", port['status'])
+            LOG.debug("new port status is %s, origin status was %s,"
+                      " => no action", port['status'], original_port_status)
 
     def notify_port_deleted(self, context, port_id):
         LOG.info("notify_port_deleted on port %s ", port_id)
@@ -385,8 +449,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
         port_bgpvpn_info = {'id': port['id'],
                             'network_id': port['network_id']}
 
-        if port['device_owner'] == const.DEVICE_OWNER_DHCP:
-            LOG.info("Port %s is DHCP, ignoring", port['id'])
+        if self._ignore_port(context, port):
             return
 
         agent_host = self._get_port_host(context, port)
@@ -454,7 +517,7 @@ class BaGPipeBGPVPNDriver(driver_api.BGPVPNDriver):
             elif rtr_itf_removed:
                 self.router_interface_removed(context, original_port)
             else:
-                self.notify_port_updated(context, port['id'])
+                self.notify_port_updated(context, port['id'], original_port)
         except Exception as e:
             LOG.exception(_LE("Error during notification processing "
                               "%(resource)s %(event)s, %(trigger)s, "
